@@ -489,7 +489,8 @@ static void write_header(FILE *sql_file, char *db_name)
 ");
     }
     fprintf(sql_file,
-	    "/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='%s%s%s' */;\n",
+	    "/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='%s%s%s' */;\n"
+	    "/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;\n",
 	    path?"":"NO_AUTO_VALUE_ON_ZERO",compatible_mode_normal_str[0]==0?"":",",
 	    compatible_mode_normal_str);
     check_io(sql_file);
@@ -518,6 +519,8 @@ static void write_footer(FILE *sql_file)
 "/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;\n"
 "/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;\n"
 "/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;\n");
+    fprintf(sql_file,
+	    "/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;\n");
     fputs("\n", sql_file);
     check_io(sql_file);
   }
@@ -858,13 +861,18 @@ static int dbConnect(char *host, char *user,char *passwd)
     DBerror(&mysql_connection, "when trying to connect");
     return 1;
   }
+  /*
+    Don't dump SET NAMES with a pre-4.1 server (bug#7997).
+  */
+  if (mysql_get_server_version(&mysql_connection) < 40100)
+    opt_set_charset= 0;
   /*
     As we're going to set SQL_MODE, it would be lost on reconnect, so we
     cannot reconnect.
   */
   sock->reconnect= 0;
-  sprintf(buff, "/*!40100 SET @@SQL_MODE='%s' */",
-	  compatible_mode_normal_str);
+  my_snprintf(buff, sizeof(buff), "/*!40100 SET @@SQL_MODE='%s' */",
+	      compatible_mode_normal_str);
   if (mysql_query_with_error_report(sock, 0, buff))
   {
     mysql_close(sock);
@@ -1072,6 +1080,7 @@ static void print_xml_row(FILE *xml_file, const char *row_name,
   check_io(xml_file);
 }
 
+
 /*
   getStructure -- retrievs database structure, prints out corresponding
   CREATE statement and fills out insert_pat.
@@ -1098,8 +1107,9 @@ static uint getTableStructure(char *table, char* db)
   if (verbose)
     fprintf(stderr, "-- Retrieving table structure for table %s...\n", table);
 
-  sprintf(insert_pat,"SET OPTION SQL_QUOTE_SHOW_CREATE=%d",
-	  (opt_quoted || opt_keywords));
+  my_snprintf(insert_pat, sizeof(insert_pat), 
+	      "SET OPTION SQL_QUOTE_SHOW_CREATE=%d",
+	      (opt_quoted || opt_keywords));
   if (!create_options)
     strmov(strend(insert_pat), "/*!40102 ,SQL_MODE=concat(@@sql_mode, _utf8 ',NO_KEY_OPTIONS,NO_TABLE_OPTIONS,NO_FIELD_OPTIONS') */");
 
@@ -1117,7 +1127,7 @@ static uint getTableStructure(char *table, char* db)
       /* Make an sql-file, if path was given iow. option -T was given */
       char buff[20+FN_REFLEN];
 
-      sprintf(buff,"show create table %s", result_table);
+      my_snprintf(buff, sizeof(buff), "show create table %s", result_table);
       if (mysql_query_with_error_report(sock, 0, buff))
       {
         safe_exit(EX_MYSQLERR);
@@ -1155,7 +1165,8 @@ static uint getTableStructure(char *table, char* db)
       check_io(sql_file);
       mysql_free_result(tableRes);
     }
-    sprintf(insert_pat,"show fields from %s", result_table);
+    my_snprintf(insert_pat, sizeof(insert_pat), "show fields from %s", 
+		result_table);
     if (mysql_query_with_error_report(sock, &tableRes, insert_pat))
     {
       if (path)
@@ -1165,11 +1176,12 @@ static uint getTableStructure(char *table, char* db)
     }
 
     if (cFlag)
-      sprintf(insert_pat, "INSERT %sINTO %s (", delayed, opt_quoted_table);
+      my_snprintf(insert_pat, sizeof(insert_pat), "INSERT %sINTO %s (", 
+		  delayed, opt_quoted_table);
     else
     {
-      sprintf(insert_pat, "INSERT %sINTO %s VALUES ", delayed,
-	      opt_quoted_table);
+      my_snprintf(insert_pat, sizeof(insert_pat), "INSERT %sINTO %s VALUES ", 
+		  delayed, opt_quoted_table);
       if (!extended_insert)
         strcat(insert_pat,"(");
     }
@@ -1196,7 +1208,8 @@ static uint getTableStructure(char *table, char* db)
               "%s: Warning: Can't set SQL_QUOTE_SHOW_CREATE option (%s)\n",
               my_progname, mysql_error(sock));
 
-    sprintf(insert_pat,"show fields from %s", result_table);
+    my_snprintf(insert_pat, sizeof(insert_pat), "show fields from %s", 
+		result_table);
     if (mysql_query_with_error_report(sock, &tableRes, insert_pat))
     {
       safe_exit(EX_MYSQLERR);
@@ -1231,10 +1244,12 @@ static uint getTableStructure(char *table, char* db)
       check_io(sql_file);
     }
     if (cFlag)
-      sprintf(insert_pat, "INSERT %sINTO %s (", delayed, result_table);
+      my_snprintf(insert_pat, sizeof(insert_pat), "INSERT %sINTO %s (", 
+		  delayed, result_table);
     else
     {
-      sprintf(insert_pat, "INSERT %sINTO %s VALUES ", delayed, result_table);
+      my_snprintf(insert_pat, sizeof(insert_pat), "INSERT %sINTO %s VALUES ",
+		  delayed, result_table);
       if (!extended_insert)
         strcat(insert_pat,"(");
     }
@@ -1291,7 +1306,7 @@ static uint getTableStructure(char *table, char* db)
       /* Make an sql-file, if path was given iow. option -T was given */
       char buff[20+FN_REFLEN];
       uint keynr,primary_key;
-      sprintf(buff,"show keys from %s", result_table);
+      my_snprintf(buff, sizeof(buff), "show keys from %s", result_table);
       if (mysql_query_with_error_report(sock, &tableRes, buff))
       {
         if (path)
@@ -1361,8 +1376,12 @@ static uint getTableStructure(char *table, char* db)
       if (create_options)
       {
 	char show_name_buff[FN_REFLEN];
-        sprintf(buff,"show table status like %s",
-		quote_for_like(table, show_name_buff));
+
+	/* Check memory for quote_for_like() */
+	DBUG_ASSERT(2*sizeof(table) < sizeof(show_name_buff));
+        my_snprintf(buff, sizeof(buff), "show table status like %s",
+		    quote_for_like(table, show_name_buff));
+
         if (mysql_query_with_error_report(sock, &tableRes, buff))
         {
 	  if (mysql_errno(sock) != ER_PARSE_ERROR)
@@ -1522,8 +1541,9 @@ static void dumpTable(uint numFields, char *table)
     my_delete(filename, MYF(0)); /* 'INTO OUTFILE' doesn't work, if
 				    filename wasn't deleted */
     to_unix_path(filename);
-    sprintf(query, "SELECT /*!40001 SQL_NO_CACHE */ * INTO OUTFILE '%s'",
-	    filename);
+    my_snprintf(query, QUERY_LENGTH, 
+		"SELECT /*!40001 SQL_NO_CACHE */ * INTO OUTFILE '%s'",
+		filename);
     end= strend(query);
 
     if (fields_terminated || enclosed || opt_enclosed || escaped)
@@ -1535,7 +1555,7 @@ static void dumpTable(uint numFields, char *table)
     end= add_load_option(end, lines_terminated, " LINES TERMINATED BY");
     *end= '\0';
 
-    sprintf(buff," FROM %s", result_table);
+    my_snprintf(buff, sizeof(buff), " FROM %s", result_table);
     end= strmov(end,buff);
     if (where || order_by)
     {
@@ -1563,8 +1583,9 @@ static void dumpTable(uint numFields, char *table)
 	      result_table);
       check_io(md_result_file);
     }
-    sprintf(query, "SELECT /*!40001 SQL_NO_CACHE */ * FROM %s",
-	    result_table);
+    my_snprintf(query, QUERY_LENGTH,
+		"SELECT /*!40001 SQL_NO_CACHE */ * FROM %s",
+		result_table);
     if (where || order_by)
     {
       query = alloc_query_str((ulong) (strlen(query) + 1 +
@@ -1662,8 +1683,9 @@ static void dumpTable(uint numFields, char *table)
         int is_blob;
 	if (!(field = mysql_fetch_field(res)))
 	{
-	  sprintf(query,"%s: Not enough fields from table %s! Aborting.\n",
-		  my_progname, result_table);
+	  my_snprintf(query, QUERY_LENGTH,
+		      "%s: Not enough fields from table %s! Aborting.\n",
+		      my_progname, result_table);
 	  fputs(query,stderr);
 	  error= EX_CONSCHECK;
 	  goto err;
@@ -1782,7 +1804,7 @@ static void dumpTable(uint numFields, char *table)
                 char *ptr= row[i], *end= ptr+ lengths[i];
                 fputs("0x", md_result_file);
                 for (; ptr < end ; ptr++)
-                  fprintf(md_result_file, "%02X", *ptr);
+		  fprintf(md_result_file, "%02X", *((uchar *)ptr));
               }
               else
                 unescape(md_result_file, row[i], lengths[i]);
@@ -1864,12 +1886,13 @@ static void dumpTable(uint numFields, char *table)
     check_io(md_result_file);
     if (mysql_errno(sock))
     {
-      sprintf(query,"%s: Error %d: %s when dumping table %s at row: %ld\n",
-	      my_progname,
-	      mysql_errno(sock),
-	      mysql_error(sock),
-	      result_table,
-	      rownr);
+      my_snprintf(query, QUERY_LENGTH,
+		  "%s: Error %d: %s when dumping table %s at row: %ld\n",
+		  my_progname,
+		  mysql_errno(sock),
+		  mysql_error(sock),
+		  result_table,
+		  rownr);
       fputs(query,stderr);
       error= EX_CONSCHECK;
       goto err;
@@ -1985,8 +2008,9 @@ static int init_dumping(char *database)
         MYSQL_ROW row;
         MYSQL_RES *dbinfo;
 
-        sprintf(qbuf,"SHOW CREATE DATABASE IF NOT EXISTS %s",
-		qdatabase);
+        my_snprintf(qbuf, sizeof(qbuf), 
+		    "SHOW CREATE DATABASE IF NOT EXISTS %s",
+		    qdatabase);
 
         if (mysql_query(sock, qbuf) || !(dbinfo = mysql_store_result(sock)))
         {
@@ -2098,10 +2122,15 @@ static void get_actual_table_name(const char *old_table_name,
 {
   MYSQL_RES  *tableRes;
   MYSQL_ROW  row;
-  char query[ NAME_LEN + 50 ];
+  char query[50 + 2*NAME_LEN];
+  char show_name_buff[FN_REFLEN];
   DBUG_ENTER("get_actual_table_name");
 
-  sprintf( query, "SHOW TABLES LIKE '%s'", old_table_name);
+  /* Check memory for quote_for_like() */
+  DBUG_ASSERT(2*sizeof(old_table_name) < sizeof(show_name_buff));
+  my_snprintf(query, sizeof(query), "SHOW TABLES LIKE %s", 
+	      quote_for_like(old_table_name, show_name_buff));
+
   if (mysql_query_with_error_report(sock, 0, query))
   {
     safe_exit(EX_MYSQLERR);
@@ -2346,8 +2375,10 @@ static const char *check_if_ignore_table(const char *table_name)
   MYSQL_ROW row;
   const char *result= 0;
 
-  sprintf(buff,"show table status like %s",
-	  quote_for_like(table_name, show_name_buff));
+  /* Check memory for quote_for_like() */
+  DBUG_ASSERT(2*sizeof(table_name) < sizeof(show_name_buff));
+  my_snprintf(buff, sizeof(buff), "show table status like %s",
+	      quote_for_like(table_name, show_name_buff));
   if (mysql_query_with_error_report(sock, &res, buff))
   {
     if (mysql_errno(sock) != ER_PARSE_ERROR)
@@ -2400,7 +2431,8 @@ static char *primary_key_fields(const char *table_name)
   uint result_length = 0;
   char *result = 0;
 
-  sprintf(show_keys_buff, "SHOW KEYS FROM %s", table_name);
+  my_snprintf(show_keys_buff, sizeof(show_keys_buff), 
+	      "SHOW KEYS FROM %s", table_name);
   if (mysql_query(sock, show_keys_buff) ||
       !(res = mysql_store_result(sock)))
   {
