@@ -417,7 +417,7 @@ row_create_prebuilt(
 	prebuilt->sel_graph = NULL;
 
 	prebuilt->search_tuple = dtuple_create(heap,
-						dict_table_get_n_cols(table));
+					2 * dict_table_get_n_cols(table));
 	
 	clust_index = dict_table_get_first_index(table);
 
@@ -745,7 +745,7 @@ run_again:
 
 		trx->op_info = "";
 
-		return(err);
+		return((int) err);
 	}
 
 	que_thr_stop_for_mysql_no_error(thr, trx);
@@ -839,7 +839,7 @@ run_again:
 
 		trx->op_info = "";
 
-		return(err);
+		return((int) err);
 	}
 
 	que_thr_stop_for_mysql_no_error(thr, trx);
@@ -954,7 +954,7 @@ run_again:
 
 		trx->op_info = "";
 
-		return(err);
+		return((int) err);
 	}
 
 	que_thr_stop_for_mysql_no_error(thr, trx);
@@ -1202,7 +1202,7 @@ run_again:
 
 		trx->op_info = "";
 
-		return(err);
+		return((int) err);
 	}
 
 	que_thr_stop_for_mysql_no_error(thr, trx);
@@ -1367,7 +1367,7 @@ row_mysql_recover_tmp_table(
 	}
 	else {
 		int	status;
-		int	namelen = strlen(table->name);
+		int	namelen = (int) strlen(table->name);
 		char*	old_name = mem_strdupl(table->name, namelen);
 		/* replace "rsql" with "#sql" */
 		old_name[ptr - table->name + 1] = '#';
@@ -1857,7 +1857,7 @@ row_drop_table_for_mysql_in_background(
 
   	trx_free_for_background(trx);
 
-	return(error);
+	return((int) error);
 }
 
 /*************************************************************************
