@@ -727,7 +727,7 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
       strmov(delimiter, DEFAULT_DELIMITER);
     else
       strmake(delimiter, argument, sizeof(delimiter) - 1);
-    delimiter_length= strlen(delimiter);
+    delimiter_length= (uint)strlen(delimiter);
     delimiter_str= delimiter;
     break;
   case OPT_LOCAL_INFILE:
@@ -1581,6 +1581,22 @@ static int reconnect(void)
   return 0;
 }
 
+static void get_current_db()
+{
+  MYSQL_RES *res;
+
+  my_free(current_db, MYF(MY_ALLOW_ZERO_PTR));
+  current_db= NULL;
+  /* In case of error below current_db will be NULL */
+  if (!mysql_query(&mysql, "SELECT DATABASE()") &&
+      (res= mysql_use_result(&mysql)))
+  {
+    MYSQL_ROW row= mysql_fetch_row(res);
+    if (row[0])
+      current_db= my_strdup(row[0], MYF(MY_WME));
+    mysql_free_result(res);
+  }
+}
 
 /***************************************************************************
  The different commands
@@ -1647,7 +1663,7 @@ static int com_server_help(String *buffer __attribute__((unused)),
   if (!connected && reconnect())
     return 1;
 
-  if ((error= mysql_real_query_for_lazy(server_cmd,strlen(server_cmd))) ||
+  if ((error= mysql_real_query_for_lazy(server_cmd,(int)strlen(server_cmd))) ||
       (error= mysql_store_result_for_lazy(&result)))
     return error;
 
@@ -1733,7 +1749,7 @@ com_help(String *buffer __attribute__((unused)),
   for (i = 0; commands[i].name; i++)
   {
     end= strmov(buff, commands[i].name);
-    for (j= strlen(commands[i].name); j < 10; j++)
+    for (j= (int)strlen(commands[i].name); j < 10; j++)
       end= strmov(end, " ");
     if (commands[i].func)
       tee_fprintf(stdout, "%s(\\%c) %s\n", buff,
@@ -1905,6 +1921,10 @@ com_go(String *buffer,char *line __attribute__((unused)))
   if (err >= 1)
     error= put_error(&mysql);
 
+  if (!error && !status.batch && 
+      (mysql.server_status & SERVER_STATUS_DB_DROPPED))
+    get_current_db();
+
   return error;				/* New command follows */
 }
 
@@ -2106,7 +2126,7 @@ print_table_data_xml(MYSQL_RES *result)
   mysql_field_seek(result,0);
 
   tee_fputs("<?xml version=\"1.0\"?>\n\n<resultset statement=\"", PAGER);
-  xmlencode_print(glob_buffer.ptr(), strlen(glob_buffer.ptr()));
+  xmlencode_print(glob_buffer.ptr(), (int)strlen(glob_buffer.ptr()));
   tee_fputs("\">", PAGER);
 
   fields = mysql_fetch_fields(result);
@@ -2595,7 +2615,7 @@ com_delimiter(String *buffer __attribute__((unused)), char *line)
     return 0;
   }
   strmake(delimiter, tmp, sizeof(delimiter) - 1);
-  delimiter_length= strlen(delimiter);
+  delimiter_length= (int)strlen(delimiter);
   delimiter_str= delimiter;
   return 0;
 }
@@ -2605,8 +2625,6 @@ static int
 com_use(String *buffer __attribute__((unused)), char *line)
 {
   char *tmp, buff[FN_REFLEN + 1];
-  MYSQL_RES *res;
-  MYSQL_ROW row;
 
   bzero(buff, sizeof(buff));
   strmov(buff, line);
@@ -2621,24 +2639,7 @@ com_use(String *buffer __attribute__((unused)), char *line)
     under our feet, for example if DROP DATABASE or RENAME DATABASE
     (latter one not yet available by the time the comment was written)
   */
-  /*  Let's reset current_db, assume it's gone */
-  my_free(current_db, MYF(MY_ALLOW_ZERO_PTR));
-  current_db= 0;
-  /*
-    We don't care about in case of an error below because current_db
-    was just set to 0.
-  */
-  if (!mysql_query(&mysql, "SELECT DATABASE()") &&
-      (res= mysql_use_result(&mysql)))
-  {
-    row= mysql_fetch_row(res);
-    if (row[0])
-    {
-      current_db= my_strdup(row[0], MYF(MY_WME));
-    }
-    (void) mysql_fetch_row(res);               // Read eof
-    mysql_free_result(res);
-  }
+  get_current_db();
 
   if (!current_db || cmp_database(charset_info, current_db,tmp))
   {
@@ -2914,9 +2915,9 @@ com_status(String *buffer __attribute__((unused)),
     MYSQL_ROW cur=mysql_fetch_row(result);
     if (cur)
     {
-      tee_fprintf(stdout, "Server characterset:\t%s\n", cur[0] ? cur[0] : "");
+      tee_fprintf(stdout, "Server characterset:\t%s\n", cur[2] ? cur[2] : "");
       tee_fprintf(stdout, "Db     characterset:\t%s\n", cur[3] ? cur[3] : "");
-      tee_fprintf(stdout, "Client characterset:\t%s\n", cur[2] ? cur[2] : "");
+      tee_fprintf(stdout, "Client characterset:\t%s\n", cur[0] ? cur[0] : "");
       tee_fprintf(stdout, "Conn.  characterset:\t%s\n", cur[1] ? cur[1] : "");
     }
     mysql_free_result(result);
@@ -3072,9 +3073,14 @@ void tee_fprintf(FILE *file, const char *fmt, ...)
 #ifdef OS2
   fflush( file);
 #endif
-  if (opt_outfile)
-    (void) vfprintf(OUTFILE, fmt, args);
   va_end(args);
+
+  if (opt_outfile)
+  {
+    va_start(args, fmt);
+    (void) vfprintf(OUTFILE, fmt, args);
+    va_end(args);
+  }
 }
 
 
