@@ -64,7 +64,7 @@ static my_bool init_state_maps(CHARSET_INFO *cs)
     else if (my_mbcharlen(cs, i)>1)
       state_map[i]=(uchar) MY_LEX_IDENT;
 #endif
-    else if (!my_isgraph(cs,i))
+    else if (my_isspace(cs,i))
       state_map[i]=(uchar) MY_LEX_SKIP;
     else
       state_map[i]=(uchar) MY_LEX_CHAR;
@@ -215,7 +215,7 @@ static int add_collation(CHARSET_INFO *cs)
 
       if (!strcmp(cs->csname,"ucs2") )
       {
-#ifdef HAVE_CHARSET_ucs2
+#if defined(HAVE_CHARSET_ucs2) && defined(HAVE_UCA_COLLATIONS)
         new->cset= my_charset_ucs2_general_uca.cset;
         new->coll= my_charset_ucs2_general_uca.coll;
         new->strxfrm_multiply= my_charset_ucs2_general_uca.strxfrm_multiply;
@@ -530,7 +530,7 @@ CHARSET_INFO *get_charset_by_name(const char *cs_name, myf flags)
   {
     char index_file[FN_REFLEN];
     strmov(get_charsets_dir(index_file),MY_CHARSET_INDEX);
-    my_error(EE_UNKNOWN_CHARSET, MYF(ME_BELL), cs_name, index_file);
+    my_error(EE_UNKNOWN_COLLATION, MYF(ME_BELL), cs_name, index_file);
   }
 
   return cs;
@@ -581,6 +581,23 @@ ulong escape_string_for_mysql(CHARSET_INFO *charset_info, char *to,
       from--;
       continue;
     }
+    /*
+     If the next character appears to begin a multi-byte character, we
+     escape that first byte of that apparent multi-byte character. (The
+     character just looks like a multi-byte character -- if it were actually
+     a multi-byte character, it would have been passed through in the test
+     above.)
+
+     Without this check, we can create a problem by converting an invalid
+     multi-byte character into a valid one. For example, 0xbf27 is not
+     a valid GBK character, but 0xbf5c is. (0x27 = ', 0x5c = \)
+    */
+    if (use_mb_flag && (l= my_mbcharlen(charset_info, *from)) > 1)
+    {
+      *to++= '\\';
+      *to++= *from;
+      continue;
+    }
 #endif
     switch (*from) {
     case 0:				/* Must be escaped for 'mysql' */
