@@ -302,7 +302,8 @@ public:
     ExtBigunsigned = NdbSqlUtil::Type::Bigunsigned,
     ExtFloat = NdbSqlUtil::Type::Float,
     ExtDouble = NdbSqlUtil::Type::Double,
-    ExtDecimal = NdbSqlUtil::Type::Decimal,
+    ExtOlddecimal = NdbSqlUtil::Type::Olddecimal,
+    ExtOlddecimalunsigned = NdbSqlUtil::Type::Olddecimalunsigned,
     ExtChar = NdbSqlUtil::Type::Char,
     ExtVarchar = NdbSqlUtil::Type::Varchar,
     ExtBinary = NdbSqlUtil::Type::Binary,
@@ -411,9 +412,20 @@ public:
         AttributeSize = DictTabInfo::a64Bit;
         AttributeArraySize = AttributeExtLength;
         return true;
-      case DictTabInfo::ExtDecimal:
-        // not yet implemented anywhere
-        break;
+      case DictTabInfo::ExtOlddecimal:
+        AttributeType = DictTabInfo::StringType;
+        AttributeSize = DictTabInfo::an8Bit;
+        AttributeArraySize =
+          (1 + AttributeExtPrecision + (int(AttributeExtScale) > 0)) *
+          AttributeExtLength;
+        return true;
+      case DictTabInfo::ExtOlddecimalunsigned:
+        AttributeType = DictTabInfo::StringType;
+        AttributeSize = DictTabInfo::an8Bit;
+        AttributeArraySize =
+          (0 + AttributeExtPrecision + (int(AttributeExtScale) > 0)) *
+          AttributeExtLength;
+        return true;
       case DictTabInfo::ExtChar:
       case DictTabInfo::ExtBinary:
         AttributeType = DictTabInfo::StringType;
