@@ -114,7 +114,8 @@ NdbColumnImpl::init(Type t)
     m_length = 1;
     m_cs = NULL;
     break;
-  case Decimal:
+  case Olddecimal:
+  case Olddecimalunsigned:
     m_precision = 10;
     m_scale = 0;
     m_length = 1;
@@ -945,6 +946,12 @@ NdbDictInterface::dictSignal(NdbApiSignal* signal,
     if(m_waiter.m_state == WAIT_NODE_FAILURE)
       continue;
 
+    if(m_waiter.m_state == WST_WAIT_TIMEOUT)
+    {
+      m_error.code = 4008;
+      DBUG_RETURN(-1);
+    }
+    
     if ( (temporaryMask & m_error.code) != 0 ) {
       continue;
     }
@@ -1176,7 +1183,8 @@ columnTypeMapping[] = {
   { DictTabInfo::ExtBigunsigned,     NdbDictionary::Column::Bigunsigned },
   { DictTabInfo::ExtFloat,           NdbDictionary::Column::Float },
   { DictTabInfo::ExtDouble,          NdbDictionary::Column::Double },
-  { DictTabInfo::ExtDecimal,         NdbDictionary::Column::Decimal },
+  { DictTabInfo::ExtOlddecimal,      NdbDictionary::Column::Olddecimal },
+  { DictTabInfo::ExtOlddecimalunsigned, NdbDictionary::Column::Olddecimalunsigned },
   { DictTabInfo::ExtChar,            NdbDictionary::Column::Char },
   { DictTabInfo::ExtVarchar,         NdbDictionary::Column::Varchar },
   { DictTabInfo::ExtBinary,          NdbDictionary::Column::Binary },
@@ -2091,8 +2099,8 @@ int
 NdbDictInterface::createIndex(NdbApiSignal* signal, 
 			      LinearSectionPtr ptr[3])
 {
-  const int noErrCodes = 1;
-  int errCodes[noErrCodes] = {CreateIndxRef::Busy};
+  const int noErrCodes = 2;
+  int errCodes[noErrCodes] = {CreateIndxRef::Busy, CreateIndxRef::NotMaster};
   return dictSignal(signal,ptr,2,
 		    1 /*use masternode id*/,
 		    100,
@@ -2116,6 +2124,8 @@ NdbDictInterface::execCREATE_INDX_REF(NdbApiSignal * signal,
 {
   const CreateIndxRef* const ref = CAST_CONSTPTR(CreateIndxRef, signal->getDataPtr());
   m_error.code = ref->getErrorCode();
+  if(m_error.code == ref->NotMaster)
+    m_masterNodeId= ref->masterNodeId;
   m_waiter.signal(NO_WAIT);  
 }
 
@@ -2212,8 +2222,8 @@ NdbDictInterface::dropIndex(const NdbIndexImpl & impl,
 int
 NdbDictInterface::dropIndex(NdbApiSignal* signal, LinearSectionPtr ptr[3])
 {
-  const int noErrCodes = 1;
-  int errCodes[noErrCodes] = {DropIndxRef::Busy};
+  const int noErrCodes = 2;
+  int errCodes[noErrCodes] = {DropIndxRef::Busy, DropIndxRef::NotMaster};
   int r = dictSignal(signal,NULL,0,
 		     1/*Use masternode id*/,
 		     100,
@@ -2240,6 +2250,8 @@ NdbDictInterface::execDROP_INDX_REF(NdbApiSignal * signal,
 {
   const DropIndxRef* const ref = CAST_CONSTPTR(DropIndxRef, signal->getDataPtr());
   m_error.code = ref->getErrorCode();
+  if(m_error.code == ref->NotMaster)
+    m_masterNodeId= ref->masterNodeId;
   m_waiter.signal(NO_WAIT);  
 }
 
