@@ -54,18 +54,18 @@ then
   c_d="$c_d   Host char(60) binary DEFAULT '' NOT NULL,"
   c_d="$c_d   Db char(64) binary DEFAULT '' NOT NULL,"
   c_d="$c_d   User char(16) binary DEFAULT '' NOT NULL,"
-  c_d="$c_d   Select_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Insert_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Update_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Delete_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Create_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Drop_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Grant_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   References_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Index_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Alter_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Create_tmp_table_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_d="$c_d   Lock_tables_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Select_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Insert_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Update_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Delete_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Create_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Drop_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Grant_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   References_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Index_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Alter_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_d="$c_d   Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
   c_d="$c_d PRIMARY KEY Host (Host,Db,User),"
   c_d="$c_d KEY User (User)"
   c_d="$c_d ) engine=MyISAM"
@@ -85,18 +85,18 @@ then
   c_h="$c_h CREATE TABLE host ("
   c_h="$c_h  Host char(60) binary DEFAULT '' NOT NULL,"
   c_h="$c_h  Db char(64) binary DEFAULT '' NOT NULL,"
-  c_h="$c_h  Select_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Insert_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Update_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Delete_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Create_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Drop_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Grant_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  References_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Index_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Alter_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Create_tmp_table_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_h="$c_h  Lock_tables_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Select_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Insert_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Update_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Delete_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Create_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Drop_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Grant_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  References_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Index_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Alter_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_h="$c_h  Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
   c_h="$c_h  PRIMARY KEY Host (Host,Db)"
   c_h="$c_h ) engine=MyISAM"
   c_h="$c_h CHARACTER SET utf8 COLLATE utf8_bin"
@@ -113,28 +113,28 @@ then
   c_u="$c_u   Host char(60) binary DEFAULT '' NOT NULL,"
   c_u="$c_u   User char(16) binary DEFAULT '' NOT NULL,"
   c_u="$c_u   Password char(41) binary DEFAULT '' NOT NULL,"
-  c_u="$c_u   Select_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Insert_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Update_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Delete_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Create_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Drop_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Reload_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Shutdown_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Process_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   File_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Grant_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   References_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Index_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Alter_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Show_db_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Super_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Create_tmp_table_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Lock_tables_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Execute_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Repl_slave_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   Repl_client_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
-  c_u="$c_u   ssl_type enum('','ANY','X509', 'SPECIFIED') DEFAULT '' NOT NULL,"
+  c_u="$c_u   Select_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Insert_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Update_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Delete_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Create_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Drop_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Reload_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Shutdown_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Process_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   File_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Grant_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   References_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Index_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Alter_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Show_db_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Super_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Execute_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Repl_slave_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   Repl_client_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
+  c_u="$c_u   ssl_type enum('','ANY','X509', 'SPECIFIED') COLLATE utf8_general_ci DEFAULT '' NOT NULL,"
   c_u="$c_u   ssl_cipher BLOB NOT NULL,"
   c_u="$c_u   x509_issuer BLOB NOT NULL,"
   c_u="$c_u   x509_subject BLOB NOT NULL,"
@@ -178,7 +178,7 @@ then
   c_f="$c_f   name char(64) binary DEFAULT '' NOT NULL,"
   c_f="$c_f   ret tinyint(1) DEFAULT '0' NOT NULL,"
   c_f="$c_f   dl char(128) DEFAULT '' NOT NULL,"
-  c_f="$c_f   type enum ('function','aggregate') NOT NULL,"
+  c_f="$c_f   type enum ('function','aggregate') COLLATE utf8_general_ci NOT NULL,"
   c_f="$c_f   PRIMARY KEY (name)"
   c_f="$c_f ) engine=MyISAM"
   c_f="$c_f CHARACTER SET utf8 COLLATE utf8_bin"
@@ -198,8 +198,8 @@ then
   c_t="$c_t   Table_name char(64) binary DEFAULT '' NOT NULL,"
   c_t="$c_t   Grantor char(77) DEFAULT '' NOT NULL,"
   c_t="$c_t   Timestamp timestamp(14),"
-  c_t="$c_t   Table_priv set('Select','Insert','Update','Delete','Create','Drop','Grant','References','Index','Alter') DEFAULT '' NOT NULL,"
-  c_t="$c_t   Column_priv set('Select','Insert','Update','References') DEFAULT '' NOT NULL,"
+  c_t="$c_t   Table_priv set('Select','Insert','Update','Delete','Create','Drop','Grant','References','Index','Alter') COLLATE utf8_general_ci DEFAULT '' NOT NULL,"
+  c_t="$c_t   Column_priv set('Select','Insert','Update','References') COLLATE utf8_general_ci DEFAULT '' NOT NULL,"
   c_t="$c_t   PRIMARY KEY (Host,Db,User,Table_name),"
   c_t="$c_t   KEY Grantor (Grantor)"
   c_t="$c_t ) engine=MyISAM"
@@ -220,7 +220,7 @@ then
   c_c="$c_c   Table_name char(64) binary DEFAULT '' NOT NULL,"
   c_c="$c_c   Column_name char(64) binary DEFAULT '' NOT NULL,"
   c_c="$c_c   Timestamp timestamp(14),"
-  c_c="$c_c   Column_priv set('Select','Insert','Update','References') DEFAULT '' NOT NULL,"
+  c_c="$c_c   Column_priv set('Select','Insert','Update','References') COLLATE utf8_general_ci DEFAULT '' NOT NULL,"
   c_c="$c_c   PRIMARY KEY (Host,Db,User,Table_name,Column_name)"
   c_c="$c_c ) engine=MyISAM"
   c_c="$c_c CHARACTER SET utf8 COLLATE utf8_bin"
@@ -328,7 +328,7 @@ then
 
   c_tz="$c_tz CREATE TABLE time_zone ("
   c_tz="$c_tz   Time_zone_id int unsigned NOT NULL auto_increment,"
-  c_tz="$c_tz   Use_leap_seconds enum('Y','N') DEFAULT 'N' NOT NULL,"
+  c_tz="$c_tz   Use_leap_seconds enum('Y','N') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,"
   c_tz="$c_tz   PRIMARY KEY TzId (Time_zone_id)"
   c_tz="$c_tz ) engine=MyISAM CHARACTER SET utf8"
   c_tz="$c_tz   comment='Time zones';"
