@@ -31,8 +31,9 @@ void item_init(void);			/* Init item functions */
 
 enum Derivation
 {
-  DERIVATION_IGNORABLE= 4,
-  DERIVATION_COERCIBLE= 3,
+  DERIVATION_IGNORABLE= 5,
+  DERIVATION_COERCIBLE= 4,
+  DERIVATION_SYSCONST= 3,
   DERIVATION_IMPLICIT= 2,
   DERIVATION_NONE= 1,
   DERIVATION_EXPLICIT= 0
@@ -61,22 +62,16 @@ class DTCollation {
 public:
   CHARSET_INFO     *collation;
   enum Derivation derivation;
-  uint nagg;    // Total number of aggregated collations.
-  uint strong;  // Number of the strongest collation.
   
   DTCollation()
   {
     collation= &my_charset_bin;
     derivation= DERIVATION_NONE;
-    nagg= 0;
-    strong= 0;
   }
   DTCollation(CHARSET_INFO *collation_arg, Derivation derivation_arg)
   {
     collation= collation_arg;
     derivation= derivation_arg;
-    nagg= 0;
-    strong= 0;
   }
   void set(DTCollation &dt)
   { 
@@ -102,6 +97,7 @@ public:
       case DERIVATION_IGNORABLE: return "IGNORABLE";
       case DERIVATION_COERCIBLE: return "COERCIBLE";
       case DERIVATION_IMPLICIT:  return "IMPLICIT";
+      case DERIVATION_SYSCONST:  return "SYSCONST";
       case DERIVATION_EXPLICIT:  return "EXPLICIT";
       case DERIVATION_NONE:      return "NONE";
       default: return "UNKNOWN";
@@ -119,7 +115,7 @@ public:
   static void *operator new(size_t size, MEM_ROOT *mem_root)
   { return (void*) alloc_root(mem_root, (uint) size); }
   static void operator delete(void *ptr,size_t size) {}
-  static void operator delete(void *ptr,size_t size, MEM_ROOT *mem_root) {}
+  static void operator delete(void *ptr, MEM_ROOT *mem_root) {}
 
   enum Type {FIELD_ITEM, FUNC_ITEM, SUM_FUNC_ITEM, STRING_ITEM,
 	     INT_ITEM, REAL_ITEM, NULL_ITEM, VARBIN_ITEM,
@@ -474,6 +470,17 @@ public:
   Item *safe_charset_converter(CHARSET_INFO *tocs);
 };
 
+class Item_null_result :public Item_null
+{
+public:
+  Field *result_field;
+  Item_null_result() : Item_null(), result_field(0) {}
+  bool is_result_field() { return result_field != 0; }
+  void save_in_result_field(bool no_conversions)
+  {
+    save_in_field(result_field, no_conversions);
+  }
+};  
 
 /* Item represents one placeholder ('?') of prepared statement */
 
@@ -552,6 +559,7 @@ public:
   bool get_time(TIME *tm);
   bool get_date(TIME *tm, uint fuzzydate);
   int  save_in_field(Field *field, bool no_conversions);
+  bool fix_fields(THD *, struct st_table_list *, Item **);
 
   void set_null();
   void set_int(longlong i, uint32 max_length_arg);
@@ -719,8 +727,9 @@ public:
   {
     DBUG_ASSERT(fixed == 1);
     int err;
+    char *end_not_used;
     return my_strntod(str_value.charset(), (char*) str_value.ptr(),
-		      str_value.length(), (char**) 0, &err);
+		      str_value.length(), &end_not_used, &err);
   }
   longlong val_int()
   {
@@ -742,7 +751,7 @@ public:
   Item *new_item() 
   {
     return new Item_string(name, str_value.ptr(), 
-    			   str_value.length(), &my_charset_bin);
+    			   str_value.length(), collation.collation);
   }
   Item *safe_charset_converter(CHARSET_INFO *tocs);
   String *const_string() { return &str_value; }
@@ -905,6 +914,7 @@ public:
   void save_org_in_field(Field *field)	{ (*ref)->save_org_in_field(field); }
   enum Item_result result_type () const { return (*ref)->result_type(); }
   enum_field_types field_type() const   { return (*ref)->field_type(); }
+  Field *get_tmp_table_field() { return result_field; }
   table_map used_tables() const		
   { 
     return depended_from ? OUTER_REF_TABLE_BIT : (*ref)->used_tables(); 
@@ -996,6 +1006,8 @@ public:
   The following class is used to optimize comparing of date and bigint columns
   We need to save the original item, to be able to set the field to the
   original value in 'opt_range'.
+  An instance of Item_int_with_ref may refer to a signed or an unsigned
+  integer.
 */
 
 class Item_int_with_ref :public Item_int
@@ -1010,6 +1022,11 @@ public:
   {
     return ref->save_in_field(field, no_conversions);
   }
+  Item *new_item()
+  {
+    return (ref->unsigned_flag)? new Item_uint(ref->name, ref->max_length) :
+                                 new Item_int(ref->name, ref->max_length);
+  }
 };
 
 
@@ -1044,9 +1061,10 @@ public:
   double val()
   {
     int err;
+    char *end_not_used;
     return (null_value ? 0.0 :
 	    my_strntod(str_value.charset(), (char*) str_value.ptr(),
-		       str_value.length(),NULL,&err));
+		       str_value.length(), &end_not_used, &err));
   }
   longlong val_int()
   { 
@@ -1315,32 +1333,32 @@ public:
 
 
 /*
-  Used to store type. name, length of Item for UNIONS & derived table
+  Item_type_holder used to store type. name, length of Item for UNIONS &
+  derived tables.
+
+  Item_type_holder do not need cleanup() because its time of live limited by
+  single SP/PS execution.
 */
 class Item_type_holder: public Item
 {
 protected:
-  Item_result item_type;
-  Item_result orig_type;
-  Field *field_example;
-public:
-  Item_type_holder(THD*, Item*, TABLE *);
+  TYPELIB *enum_set_typelib;
+  enum_field_types fld_type;
 
-  Item_result result_type () const { return item_type; }
+  void get_full_info(Item *item);
+public:
+  Item_type_holder(THD*, Item*);
+
+  Item_result result_type() const;
+  virtual enum_field_types field_type() const { return fld_type; };
   enum Type type() const { return TYPE_HOLDER; }
   double val();
   longlong val_int();
   String *val_str(String*);
-  bool join_types(THD *thd, Item *, TABLE *);
-  Field *example() { return field_example; }
-  static uint32 real_length(Item *item);
-  void cleanup()
-  {
-    DBUG_ENTER("Item_type_holder::cleanup");
-    Item::cleanup();
-    item_type= orig_type;
-    DBUG_VOID_RETURN;
-  }
+  bool join_types(THD *thd, Item *);
+  Field *make_field_by_type(TABLE *table);
+  static uint32 display_length(Item *item);
+  static enum_field_types get_real_type(Item *);
 };
 
 
