@@ -79,8 +79,6 @@ bool Item_func::agg_arg_collations(DTCollation &c, Item **av, uint count,
                                    uint flags)
 {
   uint i;
-  c.nagg= 0;
-  c.strong= 0;
   c.set(av[0]->collation);
   for (i= 1; i < count; i++)
   {
@@ -1348,11 +1346,6 @@ longlong Item_func_char_length::val_int()
 longlong Item_func_coercibility::val_int()
 {
   DBUG_ASSERT(fixed == 1);
-  if (args[0]->null_value)
-  {
-    null_value= 1;
-    return 0;
-  }
   null_value= 0;
   return (longlong) args[0]->collation.derivation;
 }
@@ -2361,7 +2354,7 @@ static user_var_entry *get_variable(HASH *hash, LEX_STRING &name,
     entry->value=0;
     entry->length=0;
     entry->update_query_id=0;
-    entry->collation.set(NULL, DERIVATION_NONE);
+    entry->collation.set(NULL, DERIVATION_IMPLICIT);
     /*
       If we are here, we were called from a SET or a query which sets a
       variable. Imagine it is this:
@@ -2419,8 +2412,8 @@ bool Item_func_set_user_var::fix_fields(THD *thd, TABLE_LIST *tables,
     and the variable has previously been initialized.
   */
   if (!entry->collation.collation || !args[0]->null_value)
-    entry->collation.set(args[0]->collation);
-  collation.set(entry->collation);
+    entry->collation.set(args[0]->collation.collation, DERIVATION_IMPLICIT);
+  collation.set(entry->collation.collation, DERIVATION_IMPLICIT);
   cached_result_type= args[0]->result_type();
   return 0;
 }
@@ -2432,7 +2425,7 @@ Item_func_set_user_var::fix_length_and_dec()
   maybe_null=args[0]->maybe_null;
   max_length=args[0]->max_length;
   decimals=args[0]->decimals;
-  collation.set(args[0]->collation);
+  collation.set(args[0]->collation.collation, DERIVATION_IMPLICIT);
 }
 
 
@@ -2641,25 +2634,25 @@ Item_func_set_user_var::update()
   case REAL_RESULT:
   {
     res= update_hash((void*) &save_result.vreal,sizeof(save_result.vreal),
-		     REAL_RESULT, &my_charset_bin, DERIVATION_NONE);
+		     REAL_RESULT, &my_charset_bin, DERIVATION_IMPLICIT);
     break;
   }
   case INT_RESULT:
   {
     res= update_hash((void*) &save_result.vint, sizeof(save_result.vint),
-		     INT_RESULT, &my_charset_bin, DERIVATION_NONE);
+		     INT_RESULT, &my_charset_bin, DERIVATION_IMPLICIT);
     break;
   }
   case STRING_RESULT:
   {
     if (!save_result.vstr)					// Null value
       res= update_hash((void*) 0, 0, STRING_RESULT, &my_charset_bin,
-		       DERIVATION_NONE);
+		       DERIVATION_IMPLICIT);
     else
       res= update_hash((void*) save_result.vstr->ptr(),
 		       save_result.vstr->length(), STRING_RESULT,
 		       save_result.vstr->charset(),
-		       args[0]->collation.derivation);
+		       DERIVATION_IMPLICIT);
     break;
   }
   case ROW_RESULT:
@@ -2877,7 +2870,10 @@ void Item_func_get_user_var::fix_length_and_dec()
     }
   }
   else
+  {
+    collation.set(&my_charset_bin, DERIVATION_IMPLICIT);
     null_value= 1;
+  }
 
   if (error)
     thd->fatal_error();
