@@ -58,6 +58,7 @@ void send_error(THD *thd, uint sql_errno, const char *err)
   uint length;
   char buff[MYSQL_ERRMSG_SIZE+2], *pos;
 #endif
+  const char *orig_err= err;
   NET *net= &thd->net;
   DBUG_ENTER("send_error");
   DBUG_PRINT("enter",("sql_errno: %d  err: %s", sql_errno,
@@ -82,6 +83,7 @@ void send_error(THD *thd, uint sql_errno, const char *err)
 	err=ER(sql_errno);	 /* purecov: inspected */
       }
     }
+    orig_err= err;
   }
 
 #ifdef EMBEDDED_LIBRARY
@@ -120,6 +122,9 @@ void send_error(THD *thd, uint sql_errno, const char *err)
   }
   VOID(net_write_command(net,(uchar) 255, "", 0, (char*) err,length));
 #endif  /* EMBEDDED_LIBRARY*/
+  if (!thd->killed)
+    push_warning(thd, MYSQL_ERROR::WARN_LEVEL_ERROR, sql_errno,
+                 orig_err ? orig_err : ER(sql_errno));
   thd->is_fatal_error=0;			// Error message is given
   thd->net.report_error= 0;
 
@@ -247,6 +252,9 @@ net_printf(THD *thd, uint errcode, ...)
   strmake(net->last_error, text_pos, length);
   strmake(net->sqlstate, mysql_errno_to_sqlstate(errcode), SQLSTATE_LENGTH);
 #endif
+  if (!thd->killed)
+    push_warning(thd, MYSQL_ERROR::WARN_LEVEL_ERROR, errcode,
+                 text_pos ? text_pos : ER(errcode));
   thd->is_fatal_error=0;			// Error message is given
   DBUG_VOID_RETURN;
 }
