@@ -695,6 +695,11 @@ fi
 # itself - note that they must be ordered by date (important when
 # merging BK trees)
 %changelog 
+* Mon Feb 14 2005 Lenz Grimmer <lenz@mysql.com>
+
+* Fixed the compilation comments and moved them into the separate build sections
+  for Max and Standard
+
 * Mon Feb 7 2005 Tomas Ulin <tomas@mysql.com>
 
 - enabled the "Ndbcluster" storage engine for the max binary
